#!/bin/bash

## Basic sanity
if [ -z "$SINGULARITY_libexecdir" ]; then
    echo "Could not identify the Singularity libexecdir."
    exit 1
fi

## Load functions
if [ -f "$SINGULARITY_libexecdir/singularity/functions" ]; then
    . "$SINGULARITY_libexecdir/singularity/functions"
else
    echo "Error loading functions: $SINGULARITY_libexecdir/singularity/functions"
    exit 1
fi

if [ -z "${SINGULARITY_ROOTFS:-}" ]; then
    message ERROR "Singularity root file system not defined\n"
    exit 1
fi

########## BEGIN BOOTSTRAP SCRIPT ##########

if ! DPKG_PATH=`singularity_which dpkg`; then
    message ERROR "dpkg is not in PATH...\n"
    exit 1
fi

if [ -z "${ISOPATH:-}" ]; then
    message ERROR "No 'ISOPath' defined in bootstrap definition'\n"
    exit 1
fi

if [ -z "${MOUNTDIRPATH:-}" ]; then
    MOUNT_DIR=/mnt/cdrom
else
    MOUNT_DIR=$MOUNTDIRPATH
fi

## Check directory existnce
if ! [ -d $MOUNT_DIR ]; then mkdir -p $MOUNT_DIR; fi

mount | grep $MOUNT_DIR > /dev/null
if [ $? -eq 0 ]; then
     message ERROR "Directory from 'MountDirPath' field of header is already in use"
     exit 1;
fi

SOURCE_DIR=$MOUNT_DIR/images/debs

mkdir -p $SINGULARITY_ROOTFS/var/lib/dpkg/updates
mkdir -p $SINGULARITY_ROOTFS/var/lib/dpkg/info
touch $SINGULARITY_ROOTFS/var/lib/dpkg/status
touch $SINGULARITY_ROOTFS/var/lib/dpkg/available

mount $ISOPATH $MOUNT_DIR

if ! [ -d $SOURCE_DIR ]; then
    message ERROR "Source directory not found."
    exit 1
else
    packages=`cat $SOURCE_DIR/DEB_LIST_BOOTSTRAP`
    for pack in $packages; do
        if [[ $pack = lfs-bootscripts* ]]; then

            mkdir ~/lfs-bootscripts
            pack_name=`find $SOURCE_DIR -name "lfs-bootscripts*"`
            dpkg-deb -R $pack_name ~/lfs-bootscripts
            cp ~/lfs-bootscripts/DEBIAN/preinst $SINGULARITY_ROOTFS/var/lib/dpkg/info/lfs-bootscripts.preinst
            rm ~/lfs-bootscripts/DEBIAN/preinst
            dpkg-deb -b ~/lfs-bootscripts ~/$pack.deb
   
            
            ln -s -r $SINGULARITY_ROOTFS/bin/bash $SINGULARITY_ROOTFS/bin/sh

            if ! eval "sudo chroot '$SINGULARITY_ROOTFS' /bin/bash -c '/var/lib/dpkg/info/lfs-bootscripts.preinst'"; then
                message ERROR "Preinst failed.\n"
                ABORT 255
            fi
            
            dpkg --unpack --force-all --admindir=$SINGULARITY_ROOTFS/var/lib/dpkg --root=$SINGULARITY_ROOTFS ~/$pack.deb
        
        else
            
            dpkg --unpack --force-all --admindir=$SINGULARITY_ROOTFS/var/lib/dpkg --root=$SINGULARITY_ROOTFS $SOURCE_DIR/$pack.deb
        
        fi
    done

    echo "Processing configuration files..."
    for config in $(find $SINGULARITY_ROOTFS/etc -name "*.dpkg-new"); do
        mv $config ${config%%.dpkg-new}
    done
 

    if ! eval "sudo chroot '$SINGULARITY_ROOTFS' /bin/bash -c 'find /var/lib/dpkg/info -name "*.postinst" | xargs -n 1 sh'"; then
        message ERROR "Postinsts failed!\n"
        ABORT 255
    fi

cat <<EOF >> $SINGULARITY_ROOTFS/etc/fstab
proc    /proc   proc    nosuid,noexec,nodev 0   0
sysfs   /sys    sysfs   nosuid,noexec,nodev 0   0
EOF
    
    rm $SINGULARITY_ROOTFS/etc/rc.d/rcsysinit.d/S99zbutton_init
    rm $SINGULARITY_ROOTFS/etc/rc.d/rcsysinit.d/S15mcst.start
    rm $SINGULARITY_ROOTFS/etc/rc.d/rcsysinit.d/S08mountfs
    rm $SINGULARITY_ROOTFS/etc/rc.d/rcsysinit.d/S07checkfs
    rm $SINGULARITY_ROOTFS/etc/rc.d/rcsysinit.d/S02check_rootfs
    rm $SINGULARITY_ROOTFS/etc/rc.d/rcsysinit.d/S04modules
    
    chmod 600 $SINGULARITY_ROOTFS/etc/shadow
    sed -i "s/change-it/container/" $SINGULARITY_ROOTFS/etc/sysconfig/network
   
    echo "Packages from iso are installed successfully!"

    umount $MOUNT_DIR

    exit 0;
fi
